#define UNIXMACH
/* #define PCMACHINE */  


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
/* #include <strings.h> */
#include <time.h>
#include <float.h>
#ifdef UNIXMACH
#include <strings.h>
#include <sys/time.h>
#else
#include <ctype.h>
#endif

#define TRUE      1
#define FALSE     0
#define HEADER    2
#define MIDDLE    3
#define LineSize  80
#define PerCent   0.01
#define MicroSec  0.001 
#define MYPI      3.141592653 
#define NOBOUND   5.432654432e+16

typedef enum {
  START=0,
  DATAIN,
  DUALALG,
  INTEG,
  INTEG2,
  ELAPSED
} tim_set;

typedef enum {
  CfcOk=0,
  CfcSpace,   /* fail to allocate required space */
  CfcIndef    /* indefinity is detected          */
} cfc_sta;

typedef enum{
  none = 0,
    maxcut,
    ecut,
    boxqp,
    maxcover,
    user
} probtype;

typedef struct { 
  int    index;  /* index of value; needed after sorting the values */ 
  double *val;    /* the vector of double pointers                  */ 
} orderVec; 

typedef enum {
  OptFound=0,
  IterLimit,    
  SysError=100,
  OutOfSpc,WriteFail,ReadFail,CholErr,
  MtxSzeErr,InvalCut,DataErr,BadData,BadInitPoint,DataInputError
} xcode;
  
typedef enum
{
  NAME =0,
  DIM  =1,
  ROWS =2,
  POBJM=3,
  POBJV=4,
  CONM =5,
  CONV =6,
  DOBJV=7,
  ENDATA
} secname;

typedef struct {
  int    laplacian;
  int    minimize;
  int    maxiter;
  int    findrank1sol;
  int    saverank1sol;
  int    dynamicrho;
  int    dynamicstep;
  int    usepotential;
  int    generatey0;
  int    save_y;
  int    matlab;
  double rgaptol;
  double alpha;
  double tolpiv;
  double bmfac;
  int    findvec;
  double rhofactor;
  int    cachesize;
  int    cacheunit;
  double densethresh;
  double dobj_stop;
  int    primal_checks;
  int    printit;
  probtype  prtype;      /* type of problem -- for rank reduction  */
} params;
  
typedef struct {
  int    nn0;
  int    *ja;
  double *an;
} spvec;

typedef struct {
  int    maxnrow;
  int    nrow;
  int    non0;
  spvec  *rows;
} smatx;

typedef struct {
  int    nrow;
  int    nnzo;
  double *diag;
  spvec  *roff;
} symat;
 
typedef struct { 
  int    nrow; 
  int    nnzo; 
  spvec  *roff; 
} syoff; 

typedef struct {
  int ndense;       /* Quantity of vectors classified as 'dense' */
  double **upd;     /* ndense vectors used in SMW update formula */ 
  double ** mat;    /* small matrix used in SMW update formula   */
  double * v1;      /* temporary vector used in SMW procedure    */
  double *coef;     /* coefficients of the matrix updates        */
  double ** mat2;   /* small matrix used to check psd of S or X  */
  double * v2;      /* temporary vector used in SMW procedure    */
  int *ndInd;       /* Index if negative diagonal elements of LDL^T */
  int ndiag;        /* Number of negative diagonal elemtents     */
  int * dind;       /* list of dense constraint indices          */
} smwTools;

typedef struct {
  int    mrow;     /* number of rows allocated                    */
  int    nrow;     /* number of rows used                         */
    
  int    snnz;     /* number of indices for nonzeros in S         */
  int    *shead;   /* position of first nonzero in row i of S     */
  int    *ssize;   /* number of non-zeros in row i of S below     */
                     /* the diagonal                                */
  int    *ssub;    /* column index buffer for non-zeros in S      */
  double *diag;    /* diagonal matrix D in the factorization      */
    
  int    unnz;     /* number of nonzeros in the upper factor      */
  int    ujnz;     /* number of column indices in the compressed  */
                     /* indices buffer ujsub                        */ 
  int    *ujbeg;   /* beginning position of indices in row i of U */
  int    *uhead;   /* position of first nonzero in row i of U     */
  int    *ujsze;   /* number of indices in row i of U             */ 
  int    *usub;    /* compressed column index buffer of U         */
  double *uval;    /* nonzero values in factor U                  */
    
  int    *perm;    /* permutation order                           */
  int    *invp;    /* inverse order of perm                       */
    
  int    nsnds;    /* number of supernodes                        */
  int    *subg;    /* index of the first column in supernode i    */
  int    ndens;    /* numer of dense rows                         */
  int    nsndn;    /* number supernodes in dense rows             */
  int    *dhead;   /* pointer first column in each dense row      */
  int    *dsub;    /* indices in dense rows                       */
  int    *dbeg;    /* beginning of column index                   */ 
  int    sdens;    /* separate dense row                          */
  int    upst;     /* specified index in uval for st-cut problem  */
} chfac;

typedef struct {
    int idep;
    int last;
    int most;
    int cure;
    int loca;
    int lowp;
    int ntot;
    
    int *head;
    int *port;
    int *fwrd;
    int *bwrd;
  } xlist;
  
typedef struct {
    int nnod;
    int nn0;
    int raft;
    int head;
    int last;
    int ntot;
    
    int *adjn;
    int *rbeg;
    int *rexs;
    int *rlen;
    int *rend;
    int *pres;
    int *succ;
  } order;

typedef struct model {
  char   ss[60];  /* problem name, data file name                     */
  char   spc[20]; /* Specification file name                          */
  char   yinput[30];/* file containing initial y vector               */
  char   youtput[30];/* file containing initial y vector              */
  char*  rtyp;    /* constraint type: E -- equality, G -- greater than*/
                  /* L -- less than                                   */ 
  int    m;       /* number of constraints                            */ 
  int    n;       /* size of the matrices C, S, X and A_i             */
  int    p;       /* number of LP variables                           */
  int    nnzo;    /* number of nonzeros in S                          */
  int    pnnz;    /* number of nonzeros in C                          */
  int    amnz;    /* number of nonzeros in A_1,...,A_m                */
  int    avnz;    /* number of nonzeros in a_1,...,a_m                */ 
  int    *useSMW; /* =1 if constraint dense and treated separately    */
    
  int    history; /* length of records for iter, alpha, relgap        */    
  int    iter[5]; /* iteration history for stepsize                   */
  int    itnow;   /* current iterate                                  */
  int    pfail;   /* iterates without new primal point                */
  double alpha[5]; /* trust region size  history                      */
  double rho;     /* penalty parameter in dual potential function     */
  double oldrho;  /* value in previous iterate                        */
  double prho;    /* rho used in primal update                        */
  double pnorm;   /* the current value of ||P||                       */
  double pnormold;/* previous value of ||P||                          */
  double potential; /* Dual Potential value                           */
  double ynorm;   /* Infinity norm of y                               */
  double gotpobj; /* True if a feasible primal matrix is found        */ 
    
  double gap;     /* the current primal dual gap                      */  
  double oldgap;  /* duality gap in previous iterate                  */
  double gaphist[5];/* history of duality gap                         */  
  double rgap;    /* the relative primal dual gap                     */    
    
  double pobj;    /* the current primal objective value               */
  double dobj;    /* the current dual objective value                 */

  double *rhs2;   /* vector A(S^{-1}) to determine step direction     */
 
    
  double *y;      /* dual variables                                   */
  double *ytemp;  /* temporary dual variables                         */
  double *yt2;    /* temporary vector                                 */
  double *dy1;    /* search direction 1 concerning y                  */
  double *dy2;    /* search direction 2                               */
  double *dy;     /* total search direction                           */
  double *s;      /* dual slack vector                                */
  double *vhat;	  /* Used to find rank 1 answer   			*/
  double *vsol;	  /* Rank 1 answer. i.e. vsol * vsol' = X	      	*/
  double *bestv;  /* Best rank 1 answer found	       			*/
  double bestObj; /* Best rank 1 objective value                      */
  double bestRObj;/* Best rank 1 objective value (randomized)         */
  double bestHObj;/* Best rank 1 objective value (from rows of X)     */
  double eigObj;  /* Rank 1 objective value (using max eigenvector)   */

  double *xmakery; /* dual vector used to create primal solution      */
  double *xmakerdy; /* step direction used to create primal solution  */
  double xmakerc; /* constant used to create primal solution          */

  double *b;      /* dual objective vector                            */
  double *cvec;   /* primal objective vector                          */

  smatx *amats;  /* constraint matrices                              */
  smatx *avecs;   /* constraint LP vectors                            */

  symat *cmat;    /* symmetric matrix in triangular format            */
  symat *smat;
    
  double **gram;  /* Large symmetric Gram matrix M                    */
  double *dgram;  /* Diagonal of the matrix M                         */
    
  chfac *sf;      /* cholesky factor for S                            */
  chfac *rf;      /* cholesky factor of (A^T dy + S)                  */
  chfac *mf;      /* cholesky factor for M                            */
    
  smwTools *smw;  /* Sherman-Morrison-Woodbury updating data          */
  int    *iw;     /* integer working buffer   (length:    )           */
  double *rw;     /* double float working buffer   (length: )         */
  double *wd;     /* double float working buffer of length n          */
  double *wd2;    /* double float working buffer of length n          */
    
  params par;     /* parameters                                       */
} sdpdat;


typedef struct {
  int     m;
  int     n;
  int     p;

  int outputargs;
  int inputargs;

  int     cnnzmax;
  int     cnnz;
  double *cval;
  int    *cir;
  int    *cjc;

  int     annzmax;
  int     annz;
  double *aval;
  int    *air;
  int    *ajc;

  int     ahatnnzmax;
  int     ahatnnz;
  double *ahatval;
  int    *ahatir;
  int    *ahatjc;

  double *b;
  double *cvec;
  double *y0;

  double *y_out;
  double *x;
  double *dy;
  double *xmakerc;

} sdpmatlabdata;


extern sdpdat  *sdat;
extern double *dsdpmaty0;
extern FILE    *fout;
extern clock_t *stim;
extern clock_t lt,vt,qt,gt;


#ifdef UNIXMACH
struct tms {
    clock_t    tms_utime;
    clock_t    tms_stime;
    clock_t    tms_cutime;
    clock_t    tms_cstime;
} tmrec;

clock_t times(struct tms *); 

#endif


#include "SDPfun.h"

#include "SDPfun2.h"



