#include "SDPdef.h"

int *iAlloc(int  len,
            char *info)
{
  int *r=NULL;
  
  if (len) 
  {
    r=(int*)calloc(len,sizeof(int));
    if (!r) ExitProc(OutOfSpc,info);
  }
  return r;
} /* iAlloc */

void iFree(int **x)
{
  int *r=*x;
  
  if (r) 
  {
    free(r);
    *x=NULL;
  }
} /* iFree */

void cFree(char **x)
{
 char *r = *x;
 /*
 printf("\n r %d &r %d",r,&r);
 getchar();
 */
   if(r)
   { free(r);
     *x = NULL;
   }

}

double *dAlloc(int  len,
               char *info)
{
  double *r=NULL;
  
  if (len) {
    r=(double*)calloc(len,sizeof(double));
    if (!r) ExitProc(OutOfSpc,info);
  }
  return r;
} /* dAlloc */

void dFree(double **x)
{
  double *r=*x;
  
  if (r) 
  {
    free(r);
    *x=NULL;
  }
} /* dFree */

smatx *mAlloc(int  nrow,
              int  nnzo,
              char *info)
{
  smatx *r;
  
  r=(smatx*)calloc(1,sizeof(smatx));
  if (!r) ExitProc(OutOfSpc,info);

    r->rows=(spvec*)calloc(nrow,sizeof(spvec));
    if (!r->rows) ExitProc(OutOfSpc,info);
    if (nnzo)
    {
      r->rows->ja=iAlloc(nnzo,info);
      r->rows->an=dAlloc(nnzo,info);
    }
  r->maxnrow=nrow;
  r->nrow=nrow;
  r->non0=nnzo;
  return r;
} /* mAlloc */



void mFree(smatx **a)
{
  smatx *r=*a;
  
  if (r) 
  {
    if (r->rows) 
	{
      iFree(&r->rows->ja);
      dFree(&r->rows->an);
      free(r->rows);
      r->rows=NULL;
    }
    
    r->maxnrow=0;
    r->nrow=0;
    free(r);
    *a=NULL;
  }
} /* mFree */


/*~~~~~~~~~~~~~~~~~~~~~*/
symat *SymAlloc(int  nrow,
                int  nnzo,
                char *info)
{
  symat *r;
  
  r=(symat*)calloc(1,sizeof(symat));
  if (!r) ExitProc(OutOfSpc,info);
  
  if (nrow)
  {
    r->diag=dAlloc(nrow,info);
      
    r->roff=(spvec*)calloc(nrow,sizeof(spvec));
    if (!r->roff)
      ExitProc(OutOfSpc,info);
    
    if (nnzo)
    {
      r->roff->ja=iAlloc(nnzo,info);
      r->roff->an=dAlloc(nnzo,info);
    }
  }
  r->nrow=nrow;
  r->nnzo=nnzo;
  
  return r;
} /* SymAlloc */

syoff *SyoAlloc(int  nrow, 
                int  nnzo, 
                char *info) 
{ 
  syoff *r; 
   
  r=(syoff*)calloc(1,sizeof(syoff)); 
  if (!r) ExitProc(OutOfSpc,info); 
   
  if (nrow) { 
    r->roff=(spvec*)calloc(nrow,sizeof(spvec)); 
    if (!r->roff) ExitProc(OutOfSpc,info); 
     
    if (nnzo) { 
      r->roff->ja=iAlloc(nnzo,info); 
      r->roff->an=dAlloc(nnzo,info); 
    } 
  } 
  r->nrow=nrow; 
  r->nnzo=nnzo; 
   
  return r; 
} /* SyoAlloc */ 
 
void SymFree(symat **a)
{
  symat *r=*a;
  
  if (r)
  {
    dFree(&r->diag);
    if (r->roff)
    {
      iFree(&r->roff->ja);
      dFree(&r->roff->an);
      free(r->roff);
      r->roff=NULL;
    }
    r->nrow=0;
    free(r);
    *a=NULL;
  }
} /* SymFree */

void SyoFree(syoff **a) 
{ 
  syoff *r=*a; 
   
  if (r)
  { 
    if (r->roff)
    { 
      iFree(&r->roff->ja); 
      dFree(&r->roff->an); 
      free(r->roff); 
      r->roff=NULL; 
    } 
    r->nrow=0; 
    free(r); 
    *a=NULL; 
  } 
} /* SyoFree */ 
 
int LvalAlloc(chfac *sf,
              char  *info)
{
  int nnz;
  
  nnz=iSum(sf->nrow,sf->ujsze);
  if ( nnz<=sf->unnz )
    return TRUE;
  
  sf->unnz=0;
  if (sf->uval) dFree(&sf->uval);
  sf->uval=dAlloc(nnz,info);
  
  sf->unnz=nnz;
  return TRUE;
} /* LvalAlloc */

chfac *CfcAlloc(int  maxrow,
                char *info)
{
  chfac *r=NULL;
  
  if (maxrow) {
    r=(chfac*)calloc(1,sizeof(chfac));
    if (!r) ExitProc(OutOfSpc,info);
    
    r->mrow =maxrow;
    r->nrow =maxrow;
      
    r->snnz =0;
    r->shead=iAlloc(maxrow,info);
    r->ssize=iAlloc(maxrow,info);
    r->ssub =NULL;
    r->diag =dAlloc(maxrow,info);
    r->unnz =0;
    r->ujnz =0;
    r->ujbeg=iAlloc(maxrow,info);
    r->uhead=iAlloc(maxrow,info);
    r->ujsze=iAlloc(maxrow,info);
    r->usub =NULL;
    r->uval =NULL;
    r->perm =iAlloc(maxrow,info);
    r->invp =iAlloc(maxrow,info);
    r->nsnds=0;
    r->subg =iAlloc(maxrow+1,info);      
  }
  return r;
} /* SchlAlloc */

void CfcFree(chfac **sf)
{
  chfac *r=*sf;
  
  if (*sf) {
    iFree(&r->shead);
    iFree(&r->ssize);
    iFree(&r->ssub);
    dFree(&r->diag);
    iFree(&r->ujbeg);
    iFree(&r->uhead);
    iFree(&r->ujsze);
    iFree(&r->usub);
    dFree(&r->uval);
    iFree(&r->perm);
    iFree(&r->invp);
    iFree(&r->subg);
    iFree(&r->dhead);
    iFree(&r->dbeg);
    iFree(&r->dsub);
    free(r);
  }
  *sf=NULL;
} /* CfcFree */

double **dPtAlloc(int  n,
                  char *info)
{
  int    i;
  double **r;
  
  r=NULL;
  if (!n) return r;
  
  r=(double **)calloc(n,sizeof(double*));
  if (!r) ExitProc(OutOfSpc,info);
  
  r[0]=dAlloc(n*(n-1)/2,info);
  for (i=1; i<n; i++)
    r[i]=r[i-1]+n-i;
  
  return r;
} /* dPtAlloc */

void dPtFree(double ***x)
{
  double **r=*x;
  
  if (r) {
    if (r[0])
      dFree(&r[0]);
    free(r);
    *x=NULL;
  }
} /* dPtFree */

smwTools* wAlloc(int nden,
                 int ncol)
{
  int       i;
  smwTools* r=NULL;
  
  r=(smwTools*)calloc(1,sizeof(smwTools));
  if (!r)
  {
    printf(" out of memory.\n");
    ShutDown();
    exit(0);
  }
  if (nden>0)
  {
    r->ndense=nden;
    r->upd=(double**)calloc(2*nden,sizeof(double*));
    if (!r->upd)
    {
      printf(" out of memory.\n");
      ShutDown();
      exit(0);
    }
    if (ncol>0)
    {
      r->upd[0]=dAlloc(2*nden*ncol,NULL);
      for (i=1; i<2*nden; i++)
        r->upd[i]=r->upd[i-1]+ncol;
    }
 
    r->mat  = (double **)calloc(2*nden,sizeof(double*));
    if (!r->mat)
    {
      printf(" out of memory.\n");
      ShutDown();
      exit(0);
    }
    else
    {
      r->mat[0]=dAlloc(2*nden*2*nden,NULL);
      for (i=1; i<2*nden; i++)
        r->mat[i]=r->mat[i-1]+2*nden;
    }

    r->mat2 =dPtAlloc(2*nden+1,NULL);
    r->v1   =dAlloc(2*nden,NULL);
    r->coef =dAlloc(2*nden,NULL);
    r->v2   =dAlloc(2*nden,NULL);
    r->ndInd=iAlloc(2*nden,NULL);
    r->dind =iAlloc(2*nden,NULL);
  }
  return r;
} /* wAlloc */

void wFree(smwTools** x)
{
  smwTools* r=*x;
  
  if (r)
  {
    if (r->ndense>0)
    {
      if (r->upd)
      {
        dFree(&(r->upd[0]));
        free(r->upd);
        r->upd=NULL;
      }
      if (r->mat){
	dFree(&(r->mat[0]));
	free(r->mat);
	r->mat=NULL;
      }
      dPtFree(&r->mat2);
      dFree(&r->v1);
      dFree(&r->coef);
      dFree(&r->v2);
      iFree(&r->ndInd);
      iFree(&r->dind);
    }
    free(r);
    *x=NULL; 
  }
} /* wFree */
