#include "SDPdef.h"

/* This file contains many of the hueristics used by this software package */


static double adjustrho(double rho,double multiple,double np){
  return ( (rho/np-0.99)*multiple+0.99 )*np;
}

void SetRhoScale(sdpdat *sdt, double *rhoscale, 
		 double gotrho)
/* This routine sets an array for possible values of rho.
 */
{
  int i,med, primal_checks=sdt->par.primal_checks;
  double np=1.0*sdt->n+sdt->p, target;

  primal_checks=max(1,sdt->par.primal_checks);
  target = max(gotrho,np+sqrt(np));

  switch (primal_checks){
  case 1:
    rhoscale[0]=np*sdt->par.rhofactor;
    break;
  case 2:
    rhoscale[0]=adjustrho(target,1.1,np);
    rhoscale[1]=adjustrho(target,0.9,np);
    break;
  case 3:
    rhoscale[0]=adjustrho(target,1.1,np);
    rhoscale[1]=adjustrho(target,0.9,np);
    rhoscale[2]=1.2*np;
    break;
  case 4:
    rhoscale[0]=adjustrho(target,1.2,np);
    rhoscale[1]=adjustrho(target,1.0,np);
    rhoscale[2]=adjustrho(target,0.8,np);
    rhoscale[3] = max(0.8*target,1.2*np);
    if (rhoscale[3] < rhoscale[0] && rhoscale[3] > rhoscale[2])
      rhoscale[3]=1.5*np;
    break;
  case 5:
    rhoscale[0]=adjustrho(target,1.2,np);
    rhoscale[1]=adjustrho(target,1.1,np);
    rhoscale[2]=adjustrho(target,0.9,np);
    rhoscale[3]=adjustrho(target,0.8,np);
    rhoscale[4]=1.2*np;
    break;
  default:
    med=(int) primal_checks/2 - 1;
    rhoscale[med]=target;
    for (i=med+1; i< primal_checks; i++){
      rhoscale[i]=adjustrho(rhoscale[i-1],0.94,np);
    }
    for (i=med-1; i>= 0; i--){
      rhoscale[i]=adjustrho(rhoscale[i+1],1.06,np);
    }
    rhoscale[primal_checks-1]=1.2*np;
    break;
  }
  
  return;

}


double GetDualRho(sdpdat *sdt){
/* This heuristic determined the value of rho used at each iteration.
   The dynamic choice of rho is based upon the self-concordant barrier 
   parameter and the recent improvement of the relative gap.              */

  double np = 1.0*sdt->n+sdt->p;
  double rhotemp=1.2*np;
  double pnorm0;
  
  if (sdt->par.rhofactor > 1.0)
    return (np*sdt->par.rhofactor);


  if (sdt->prho > np) { /* Found a value for rho in primal problem */
    rhotemp=sdt->prho*1.0;
  } else if (sdt->gotpobj == FALSE) {
    rhotemp=1.2*np;
  } else if (sdt->itnow==1){
    rhotemp=1.2*np;
  } else if ( sdt->pfail <= 2){
    rhotemp = sdt->oldrho;
  }else if ( fabs(sdt->pnorm/sdt->pnormold - 1.0) < 0.01) {
    rhotemp = 0.93*sdt->oldrho;
  } else {
    rhotemp = sdt->oldrho;
  }


  if      (sdt->rgap < 0.000001) rhotemp=min(rhotemp,10.0*np);
  else if (sdt->rgap < 0.00001) rhotemp=min(rhotemp,5.0*np);
  else if (sdt->rgap < 0.0001) rhotemp=min(rhotemp,4.0*np);
  else if (sdt->rgap < 0.005) rhotemp=min(rhotemp,3.0*np);
  else if (sdt->rgap < 0.05) rhotemp=min(rhotemp,2.0*np);
  else if (sdt->rgap < 1.0) rhotemp=min(rhotemp,1.6*np);
  else if (sdt->rgap < 1.5) rhotemp=min(rhotemp,1.4*np);
  else rhotemp=min(rhotemp,1.2*np);
  
  if (sdt->itnow > 0){
    AddVecs(rhotemp/sdt->gap,sdt->dy1,-1.0,sdt->dy2,sdt->dy, sdt->m);
    AddVecs(rhotemp/sdt->gap,sdt->b,-1.0,sdt->rhs2,sdt->yt2, sdt->m);
    pnorm0 = sqrt( fabs(Dot(sdt->yt2,sdt->dy,sdt->m)));
    if ( pnorm0 > sqrt(np)/10.0+1.0 && sdt->rgap > 0.01 ) 
      rhotemp=min(rhotemp,1.2*np);
  }

  rhotemp = max(rhotemp, np+sqrt(np)/4.0);

  return (rhotemp);
}



void GetStepSize( sdpdat *sdt, int itnow, int attempt, int type ){
/* This heuristic determinines the radius of the trust region used in
   this iteration 'itnow'.  This dynamic selection of radius 'alpha' is
   based upon the success of recent trust regions.  That is, if one
   radius has been successful for several iterations, we try a larger
   radius.  If one radius is too large, we reduce the size of it.       */

  int i;
  double newalpha, lowalpha=sdt->par.alpha, alphaprev = sdt->alpha[0];
  double rad = sqrt(1.0*sdt->n + sdt->p)/2.0 + 1.0;


  if (sdt->par.dynamicstep == FALSE){
    if (attempt==0) newalpha = lowalpha;
    else newalpha=alphaprev * 0.8;
  }
  else /* if (sdt->par.dynamicstep > 0) */ { 
    /* Try full Newton step.  If unsuccessful backtrack  */
    if (itnow < 1) newalpha = lowalpha;
    else if (itnow == 1 && sdt->gotpobj == FALSE && attempt==0) 
      newalpha = 0.95*min(sdt->pnorm,1.0);
    else if (attempt==0){newalpha = min(max(0.95*sdt->pnorm,0.10),rad);}
    else /* if (itnow == itprev) */ {newalpha = alphaprev*0.8;}
    
    /* An ellipsoid radius less than 1, barring numerical 
       difficulties, should always work. */
    if (attempt>0 && newalpha < lowalpha && alphaprev > lowalpha){
      newalpha = lowalpha;
    }
  }
  
  
  /* Record information */
  
  for (i = sdt->history - 1; i > 0; i--){
    sdt->alpha[i] = sdt->alpha[i-1];
    sdt->iter[i] = sdt->iter[i-1];
  }
  sdt->iter[0] = itnow;
  sdt->alpha[0] = newalpha;
}



int FindDenseMatrix(sdpdat *sdt){
/* This hueristic tries to maintain the sparsity of the dual matrix
   by detecting dense constraint matrices.                          */

  int i, denseConstraints=0, m = sdt->m;
  
  for (i=0; i<m; i++){
     if (sdt->amats->rows[i].nn0 > max(1,(sdt->n)*sdt->par.densethresh  )){  
       /* More than half of the vector is nonzero */
       sdt->useSMW[i]=1;
       denseConstraints ++;
     }
     else{
       sdt->useSMW[i]=0;
     }
   }

  if (m - denseConstraints < sdt->n){
    for (i=0;i<m;i++){
      sdt->useSMW[i] = 0;
    }
    denseConstraints = 0;
  }
  return denseConstraints;

}

