# include "SDPdef.h"
  

static void LaPlacian(symat *mat, int n){
/* Create Laplacian Matrix */
  int i,j,k;
  double rowsum;

  for (i=0; i<n; i++){
    mat->diag[i] /= -4.0;
    for (j=0; j<mat->roff[i].nn0; j++){
      mat->roff[i].an[j] /=-4.0;
    }
  }

  for (i=0;i<n; i++){
    rowsum=0.0;
    for (j=0; j<i; j++){
      k=0;
      while ((k<mat->roff[j].nn0) && (mat->roff[j].ja[k] != i)){
	k++;
      }
      if ((k<mat->roff[j].nn0)&&(mat->roff[j].ja[k] == i)){
	rowsum = rowsum + mat->roff[j].an[k];
      }
    }
    for (j=0; j<mat->roff[i].nn0; j++){
      rowsum = rowsum + mat->roff[i].an[j];
    }
    mat->diag[i] -= rowsum;
   }
}

static void NegativeObjective(symat *cmat, int n, double * cvec, int p){
  int i,j;
  
  for (i=0; i<n; i++){
    cmat->diag[i] = -(cmat->diag[i]);
    for (j=0; j<cmat->roff[i].nn0; j++){
      cmat->roff[i].an[j] = -(cmat->roff[i].an[j]);
    }
  }
  
  for (i=0; i<p; i++ ){
    cvec[i] = -cvec[i];
  }
}

static void Initial_y(sdpdat *sdt){
/* This routine selects an initial feasible dual vector y.  It
   assumes the first n constraints concern the diagonal matrix of X
   and applies the Gerschgorin Circle Theorem to find an initial point */

  int i,j,k,ind;
  double temp,diag;

  for (i=0;i<sdt->m; i++){ sdt->y[i] = - (1.0) ; }

  for (i=0;i<sdt->m;i++){
    /* Initialize dual matrix */
    /* Look for constraints that bound the diagonal of the primal atrix and
       apply Gerschgorin's Theorem. */
    if (sdt->amats->rows[i].nn0 == 1  
	/* && sdt->aa.amat[i].ja[0] == i  Part2 optional */){
      ind = sdt->amats->rows[i].ja[0];
      diag = sdt->amats->rows[i].an[0];
      temp = sdt->cmat->diag[ind];
      for (j=0; j<ind; j++){
        k=0;
        while ((k<sdt->cmat->roff[j].nn0) && 
               (sdt->cmat->roff[j].ja[k] != ind)){
          k++;
        }
        if ((k<sdt->cmat->roff[j].nn0) && (sdt->cmat->roff[j].ja[k] == ind)){
          temp -= fabs( sdt->cmat->roff[j].an[k] );
        }
      }
      for (j=0; j<sdt->cmat->roff[ind].nn0; j++){
        temp -= fabs( sdt->cmat->roff[i].an[j] );
      }
      sdt->y[ind] = - (max(-temp,0.0)+ sdt->n/1000.0 + 1.0 ) / (diag*diag) ; 
      /* sdt->y[ind] = - (max(-temp,0.0)+ 1.0 ) / (diag*diag) ; 
         sdt->y[ind] = - (max(-temp,0.0)+ sdt->n ) / (diag*diag) ; */ 
    }

    /* Initialize dual vector */
    if (sdt->avecs->rows[i].nn0 == 1){
      ind = sdt->avecs->rows[i].ja[0];
      diag =(sdt->avecs->rows[i].an[0]);
      temp = sdt->cvec[ind];
      if (temp - diag * sdt->y[i] <= 0){
        sdt->y[i] = (temp - 1.0)/diag;
      }
    }

  }

}
 
/*########### Read and count data here ##############*/
static int ReadInitialPoint(sdpdat* st,char* filename)
{
  FILE   *fp;
  int i,m=st->m,count=0;

  fp = fopen(filename,"r");
  if(!fp)
  { printf("\n Cannot open file containing initial dual solution %s",filename);
    return 1;
  }

  for(i=0;(i < m) &&(!feof(fp));i++)
  {
    if(fscanf(fp,"%lf",&st->y[i] ) ){
      count++;
    }
  } 

  if (count < m){
    printf("Error reading initial y vector: \n");
    printf("  Expecting %d entries but read only %d entries \n",m,count);
  }
  
  fclose(fp);

  if(count == st->m ) return 1;
  else return 0;
  
} /* ReadUserY */



int InitialPoint(sdpdat *sdt){

  int i,m = sdt->m, n=sdt->n,nnz;
  double ratio;

  sdt->rhs2= dAlloc(m, "Setup");
  sdt->y = dAlloc(m,"Setup");
  sdt->ytemp = dAlloc(m,"Setup");
  sdt->yt2 = dAlloc(m,"Setup");
  sdt->dy1 = dAlloc(m, "Setup");
  sdt->dy2 = dAlloc(m, "Setup");
  sdt->dy = dAlloc(m, "Setup");
  sdt->xmakery = dAlloc(m,"Setup");
  sdt->xmakerdy = dAlloc(m,"Setup");
  
  sdt->s = dAlloc(sdt->p, "Setup");
  sdt->wd = dAlloc(n,"Setup");
  sdt->wd2 = dAlloc(n,"Setup");
   sdt->vhat = dAlloc(n,"Setup");
   sdt->vsol = dAlloc(n,"Setup");
   sdt->bestv = dAlloc(n,"Setup");
   
   sdt->history = 4;
      
   /* Allocate sdt->smw in CreateS.c, 
      Allocate sdt->mf in
      Allocate sdt->sf in
      Allocate sdt->cmat, cvec, avecs, aa, b, rtype,  in DataIn() 
      */
 
   for (i=0; i<m; i++){
     sdt->y[i]=0.0;
     sdt->rhs2[i]=0.0;
     sdt->ytemp[i]=0.0;
     sdt->dy1[i]=0.0;
     sdt->dy2[i]=0.0;
     sdt->dy[i]=0.0;
     sdt->dgram[i]=0.0;
   }

   if (sdt->par.minimize == FALSE){ 
    NegativeObjective(sdt->cmat, sdt->n, sdt->cvec, sdt->p);  
    if (sdt->par.dobj_stop < 1e15)
      sdt->par.dobj_stop = -sdt->par.dobj_stop;
   }
  if (sdt->par.laplacian == TRUE)  
    LaPlacian(sdt->cmat, sdt->n);

  if (sdt->par.matlab==4 || sdt->par.matlab==6){
    DoubleCopy(dsdpmaty0, sdt->y, sdt->m);
  } else if (sdt->par.generatey0 == TRUE){ 
    Initial_y(sdt);
  } else if(ReadInitialPoint(sdt, sdt->yinput)==0)
    ExitProc(DataErr,"Length of initial y does not match no. of constaints ");


  if (sdt->par.primal_checks >= 2){
    nnz=2* 1000; /* ((sdt->sf->unnz)+n); */
    ratio=(n*m*m*1.0+nnz*m*1.0+m*m*m/3.0)/(m*nnz*1.0+n*nnz/3.0)/15.0 + 0.5;
    sdt->par.primal_checks = (int) min( max(ratio,4.0), 10.0);
  }
  
  sdt->par.primal_checks = 4;

  sdt->gotpobj = FALSE;
  sdt->pobj=0.0;
  sdt->dobj=Dot(sdt->b, sdt->y, sdt->m);
  sdt->gap = sdt->pobj - sdt->dobj;
  sdt->rgap = sdt->gap / (max(fabs(sdt->dobj),fabs(sdt->pobj)) + 1);
  sdt->rgap=2.0;

  sdt->bestObj = sdt->bestHObj = sdt->bestRObj = sdt->eigObj = NOBOUND;

  sdt->alpha[0] = 0.0;
  sdt->rho=0.0;
  sdt->pnorm=1.0e-4;
  sdt->ynorm=0; 
  for (i=0;i<sdt->m;i++) sdt->ynorm=max(fabs(sdt->y[i]),sdt->ynorm);

  for (i=0; i<sdt->history; i++){
    sdt->gaphist[i]=sdt->gap;
    sdt->alpha[i] = 0.0;
    sdt->iter[i] = 0;
  }

  NumChol(sdt, sdt->y, FALSE);

  if (LPVariables(sdt, sdt->y, sdt->s) <= 0) {
    printf(" Infeasible initial dual LP variable \n");
    return 1;
  }
  if (CheckPSD(sdt,sdt->y,sdt->sf) >0 ) {
    printf(" Infeasible initial dual matrix \n");
    return 1;
  }
  if (sdt->smw->ndense == 0){
    sdt->potential = EvalPotential(sdt,sdt->gap);
  }

  return 0;
}


  
void Initial_X(sdpdat *sdt, int *gotpobj)
/* This routine tries to find an initial feasible primal point.
   By using very large barrier parameters 'mu', an initial primal
   point should be found.
*/
{
  int i, psd = FALSE, notPSD= 1, useSMW = sdt->smw->ndense;
  double newgap, mr1, mr2, mu = 1, minmu, muscale[3];

  /* Calculate the minimum rho needed to improve objective */
  mr1 = Dot(sdt->dy1,sdt->b,sdt->m);
  mr2 = Dot(sdt->dy2,sdt->b,sdt->m);
  if (mr2>0){
    minmu = mr1/mr2;
  } else if (mr2 < 0) {
    minmu = -mr1/mr2;
  } else {
    minmu = 1;
  }

  mu= max(1e-8, minmu);

  muscale[0] = 0.9*mu;
  muscale[1] = 1.1*mu;
  muscale[2] = 2*mu;

  for (i=0; i< 3; i++){

    mu = muscale[i];
    AddVecs(1.0/mu,sdt->dy1,-1.0,sdt->dy2,sdt->dy, sdt->m);
    newgap = (Dot(sdt->dy,sdt->rhs2,sdt->m)+(sdt->n+sdt->p))*(mu);
    AddVecs(1.0,sdt->y,-1.0,sdt->dy,sdt->ytemp, sdt->m);
    /* Check if vector x is pos.   */
    if (LPVariables(sdt, sdt->ytemp, sdt->s) > 0){
      /* Check if X is pos def.      */
      if (useSMW){
	NumChol(sdt, sdt->ytemp, FALSE);
	notPSD = CheckPSD(sdt,sdt->ytemp,sdt->sf);
      }
      else{
	if (NumChol(sdt, sdt->ytemp, TRUE)==TRUE ) {notPSD = FALSE;}
      }
      if (notPSD > 0) psd = FALSE; else psd = TRUE;
      
      if ( psd == TRUE){
	/* if new primal point found   */
	DoubleCopy(sdt->y, sdt->xmakery, sdt->m);
	DoubleCopy(sdt->dy, sdt->xmakerdy, sdt->m);
	sdt->xmakerc = mu;
	sdt->gap = newgap;
	sdt->pobj = sdt->dobj + newgap;
	*gotpobj = TRUE; sdt->gotpobj=TRUE;
	return;
      }
    }
    
  }

  if (sdt->itnow==1 || (psd == FALSE && sdt->dobj+newgap>sdt->pobj) ){
      sdt->gap = newgap;
      sdt->pobj = sdt->dobj + newgap;
  }
}



