#include "SDPdef.h"

static void InitParams(params *par)
{
  par->maxiter     =200;
  par->laplacian   =FALSE;
  par->minimize    =TRUE;
  par->rhofactor   =0;
  par->findrank1sol=FALSE;
  par->saverank1sol=FALSE;
  par->dynamicrho  =FALSE;
  par->dynamicstep =TRUE;
  par->usepotential=FALSE;
  par->rgaptol     =1.0e-02;
  par->alpha       =0.90;
  par->tolpiv      =1.0e-13;
  par->bmfac       =1.0;
  par->matlab      =0;
  par->cachesize   =256;
  par->cacheunit   =1000;
  par->densethresh =0.5;
  par->prtype      =none;
  par->generatey0  =TRUE;
  par->dobj_stop   = 1.0e20;
  par->primal_checks = 4;
  par->printit     = FALSE;
  par->save_y      = FALSE;
} /* SetParams */

static int LocParam(char *ss, char *ss2)
{
  int i,k,id;
  
  k =strlen(ss);
  id=k;
  for (i=0; i<k; i++) 
  {
    if (ss[i]==':') id=i;

    ss2[i]=(char)tolower(ss[i]);

  }
 
  return id;
} /* LocParam */

void SetParams(sdpdat *sdt)
{
  int  i=0,k,ival;
  FILE *fp;
  char ss2[80],sslower[80],sold[80]="",*sstemp,*ss,
    paramtitle[][60]={
      "rank_reduction_type",    "laplacian",
      "maximum_iterations", "save_rank_one_sol",
      "dynamic_rho", "variable_steps",
      "stopping_tolerance", "pivoting_tolerance",
      "trust_region_radius(0,1)","cache_memory_size",
      "cache_memory_unit",  "min_or_max",
      "initial_primal_objective", "penalty_factor",
      "rho_factor", "rank_one_method",
      "dense_constraint_thres", "use_potential_function",
      "initial_dual_solution", "save_dual_sol",
      "dual_objective_stop","print_transcript"
       };
  double rval;

  params *par=&sdt->par;

  sstemp = calloc(256,1);
  ss     = calloc(80,1);
  if( (!sstemp)  ) { printf("\n Out of memory");
  exit(0);}

  InitParams(par);

  
  if (*(sdt->spc))
  {
    fp=fopen(sdt->spc,"r");
    if (!fp) ExitProc(ReadFail,sdt->spc);
  }
  else
  {
    fp=fopen("COPLSDP.spc","r");
    if (!fp)
    {
      return;
    }
  }


  while (!feof(fp))
  {
    fgets(sstemp,80,fp);
    ss = sstemp; 

    if (strstr(ss,"%")){
      continue;
    }
    for (i=0; i<80; i++)
      sold[i]=ss[i];

    if (*ss=='*' ||*ss=='\n'|| *ss=='\0') 
      continue;


    k=LocParam(ss,sslower)+1;
    
    for (i=0; i<23; i++)
	{
      
      if (strstr(sslower,paramtitle[i]))
        break;
    }
    switch (i) {
    case 0:
      if (strstr(sslower+k,"maxcut")) par->prtype = maxcut;
      else if (strstr(sslower+k,"ecut")) par->prtype = ecut;
      else if (strstr(sslower+k,"boxqp")) par->prtype = boxqp;
      else if (strstr(sslower+k,"maxcover")) par->prtype = maxcover;
      else if (strstr(sslower+k,"stable")) par->prtype = maxcover;
      else if (strstr(sslower+k,"user")) par->prtype = user;
      break;
    case 1:
      if (strstr(sslower+k,"yes"))
	par->laplacian=TRUE;
      else if (strstr(sslower+k,"no"))
	par->laplacian=FALSE;
      break;
    case 2:
      ival=atoi(sslower+k);
      if (ival>0) par->maxiter=ival;
      break;
    case 3:
      if (strstr(sslower+k,"yes"))
	par->saverank1sol=TRUE;
      else if (strstr(sslower+k,"no"))
	par->saverank1sol=FALSE;
      break;
    case 4:
      if (strstr(sslower+k,"on"))
	par->dynamicrho=TRUE;
      else if (strstr(sslower+k,"off"))
	par->dynamicrho=FALSE;
      break;
    case 5:
      if (strstr(sslower+k,"on"))
	par->dynamicstep=TRUE;
      else if (strstr(sslower+k,"off"))
	par->dynamicstep=FALSE;
      break;
    case 6:
      rval=atof(sslower+k);
      if (rval>0.0) par->rgaptol=rval;
      break;
    case 7:
      rval=atof(sslower+k);
      if (rval>0.0) par->tolpiv=rval;
      break;
    case 8:
      rval=atof(sslower+k);
      if ((rval>0.0) && (rval<1.0)) par->alpha=rval;
      break;
    case 9:
      ival=atoi(sslower+k);
      par->cachesize=ival;
      break;
    case 10:
      ival=atoi(sslower+k);
      if (ival>0) par->cacheunit=ival;
      break;
    case 11:
      if (strstr(sslower+k,"min"))
	par->minimize=TRUE;
      else if (strstr(sslower+k,"max"))
	par->minimize=FALSE;
      break;
    case 12:
      break;
    case 13:
      rval=atof(sslower+k);
      par->bmfac=max(rval,0.01);
      break;
    case 14:
      rval=atof(sslower+k);
      par->rhofactor=max(rval,1.0001);
      par->primal_checks=1;
      break;
    case 15:
      if (strstr(sslower+k,"no"))
	par->findrank1sol=FALSE;
      else if (strstr(sslower+k,"both"))
	par->findrank1sol=3;
      else if (strstr(sslower+k,"all"))
	par->findrank1sol=3;
      else if (strstr(sslower+k,"rand"))
	par->findrank1sol=4;
      else if (strstr(sslower+k,"xrow"))
	par->findrank1sol=2;
      else if (strstr(sslower+k,"one"))
	par->findrank1sol=1;
      else if (strstr(sslower+k,"yes"))
	par->findrank1sol=3;
      break;
    case 16:
      rval = atof(sslower+k);
      if (rval < 0.2) rval=0.2;
      par->densethresh = rval;
      break;
    case 17:
      if (strstr(sslower+k,"yes"))
	par->usepotential=TRUE;
      else if (strstr(sslower+k,"no"))
	par->usepotential=FALSE;
      break;
    case 18:
      if (strstr(sslower+k,"auto")){
	par->generatey0=TRUE; 
      }
      else{
	par->generatey0=FALSE;
	sscanf(ss,"%s : %s",ss2,sdt->yinput);
      }
      break;
    case 19:
      if (strstr(sslower+k,"no"))
	par->save_y = FALSE;
      else if (strstr(sslower+k,"yes")){
	par->save_y = TRUE;
	sprintf(sdt->youtput,"%s.ysol",sdt->ss);
      }
      else{
	par->save_y=TRUE;
	sscanf(ss,"%s : %s",ss2,sdt->youtput);
      }
      break;
    case 20:
      rval=atof(sslower+k);
      par->dobj_stop = rval;
      break;
    case 21:
      if (strstr(sslower+k,"yes"))
	par->printit = TRUE;
      if (strstr(sslower+k,"true"))
	par->printit = TRUE;
      break;
    default:
      break;
    }
  }

  free(sstemp);

  fclose(fp);
} /* SetParams */


void PrintParams(sdpdat *sdt){

  params *par=&(sdt->par);
  char  paramtitle[][60]={
    "rank_reduction_type(maxcut/ecut/maxcover/user)",    
    "create_laplacian(no/yes)",
    "maximum_iterations", "save_rank_one_solution(no/yes)",
    "dynamic_rho", "variable_steps",
    "stopping_tolerance", "pivoting_tolerance",
    "trust_region_radius(0,1)","cache_memory_size",
    "cache_memory_unit",  "min_or_max(min/max)",
    "initial_primal_objective", "penalty_factor",
    "rho_factor", "rank_one_method(no/xrow/rand/one/all)",
    "dense_constraint_threshold", "use_potential_function",
    "initial_dual_solution(automatic/filename)", "save_dual_solution(no/yes)",
    "dual_objective_stop","print_transcript_to_file(yes/no)"
  };
  
  if (fout){
    FprintHeader();
    
    fprintf(fout,"--------\n");
    fprintf(fout,"SPC FILE BEGIN\n\n");
 
    fprintf(fout,"%s : %4.2e \n",paramtitle[6],par->rgaptol);
    if (par->minimize  == FALSE) 
      fprintf(fout,"%s : max \n",paramtitle[11]);
    if (par->minimize  == TRUE) 
      fprintf(fout,"%s : min \n",paramtitle[11]);
    if (par->maxiter != 200)
      fprintf(fout,"%s : %d \n",paramtitle[2],par->maxiter);
    if (par->laplacian == TRUE) 
      fprintf(fout,"%s : yes \n",paramtitle[1]);
    
    if (par->prtype == maxcut && par->findrank1sol != FALSE)
      fprintf(fout,"%s : maxcut \n",paramtitle[0]);
    if (par->prtype == boxqp && par->findrank1sol != FALSE)
      fprintf(fout,"%s : boxqp \n",paramtitle[0]);
    if (par->prtype == ecut && par->findrank1sol != FALSE)
      fprintf(fout,"%s : ecut \n",paramtitle[0]);
    if (par->prtype == user && par->findrank1sol != FALSE)
      fprintf(fout,"%s : user \n",paramtitle[0]);
    if (par->prtype == maxcover && par->findrank1sol != FALSE)
      fprintf(fout,"%s : maxcover \n",paramtitle[0]);

    if (par->rhofactor  > 1) 
      fprintf(fout,"%s : %4.5f \n",paramtitle[14],par->rhofactor);
    if (par->saverank1sol== TRUE && par->findrank1sol != FALSE)      
      fprintf(fout,"%s : yes \n",paramtitle[3]);
    if (par->usepotential!=FALSE)
      fprintf(fout,"%s : yes \n",paramtitle[17]);
    if (par->alpha       !=0.90)
      fprintf(fout,"%s : %4.2f \n",paramtitle[8],par->alpha);
    if (par->tolpiv      !=1.0e-13)
      fprintf(fout,"%s : %4.2e \n",paramtitle[7],par->tolpiv);
    if (par->bmfac       !=1.0)
      fprintf(fout,"%s : %4.2e \n",paramtitle[13],par->bmfac);
    if (par->cachesize   !=256)
      fprintf(fout,"%s : %4d \n",paramtitle[9],par->cachesize);
    if (  par->cacheunit   !=1000)
      fprintf(fout,"%s : %4d \n",paramtitle[10],par->cacheunit);
    if ( par->densethresh != 0.5)
      fprintf(fout,"%s : %4.2f \n",paramtitle[16],par->densethresh);

    if (par->findrank1sol == 1)
      fprintf(fout,"%s : one \n",paramtitle[15]);
    if (par->findrank1sol == 2)
      fprintf(fout,"%s : xrow \n",paramtitle[15]);
    if (par->findrank1sol == 4)
      fprintf(fout,"%s : rand \n",paramtitle[15]);
    if (par->findrank1sol == 3)
      fprintf(fout,"%s : all \n",paramtitle[15]);
    if (par->generatey0  != TRUE)
      fprintf(fout,"%s : %s \n",paramtitle[18],sdt->yinput);
    if (par->generatey0  == TRUE)
      fprintf(fout,"%s : automatic \n",paramtitle[18]);
    if (par->dobj_stop   < 1.0e19 && par->minimize == TRUE)
      fprintf(fout,"%s : %4.2f \n",paramtitle[20],par->dobj_stop);
    if (par->dobj_stop   < 1.0e19 && par->minimize == FALSE)
      fprintf(fout,"%s : %4.2f \n",paramtitle[20],-par->dobj_stop);
    if (par->save_y      != FALSE) 
      fprintf(fout,"%s : yes \n",paramtitle[19]);

    
    fprintf(fout," \nSPC FILE END\n");
    fprintf(fout,"--------\n");
  }

}
