#include "SDPdef.h"

void PrintHeader(void)
{
  char Title[8][60]={
    " ==========================================\n",
    " Computational Optimization Program Library\n",
    "                                           \n",
    "     (  C  O  P  L  )     Version 2.0      \n",
    "                                           \n",
    " Computational Optimization Lab.           \n",
    " The University of Iowa (1998)             \n",
    " ==========================================\n"};
  int i;

  printf("\n\n\n");
  for (i=0; i<8; i++)
    printf("%s", Title[i]);
  printf("\n *************************** COPL STARTS ");
  printf("***************************\n\n");

  return;
} /* PrintHeader */

int FprintHeader(void)
{
  if (fout){
    fprintf(fout,"========================\n");
    fprintf(fout,"C O P L  2.0  (May 1998)\n");
    fprintf(fout,"========================\n\n");
  }
  return TRUE;
} /* FprintHeader */


int LeftDots(char *ss,int len)
{
  char sz[80];
  int  i,j,k;
  
  k=strlen(ss);
  j=len-k;
  
  strcpy(sz,ss);
  for (i=0; i<j; i++)
    ss[i]='.';
  for (i=j; i<len; i++)
    ss[i]=sz[i-j];
  ss[len]='\0';
  return TRUE;
}


void PrintTitle(sdpdat *sdt,
	        double dens, double dens2)
{
  char ss[80];
  
  sprintf(ss," %d",sdt->n);
  LeftDots(ss,55);
  printf("\n dimension n %s\n",ss);
  if (fout) fprintf(fout,"\n dimension n %s\n",ss);
  
  sprintf(ss," %d",sdt->p);
  LeftDots(ss,55);
  printf(" dimension p %s\n",ss);
  if (fout) fprintf(fout," dimension p %s\n",ss);
  
  sprintf(ss," %d",sdt->m);
  LeftDots(ss,55);
  printf(" dimension m %s\n",ss);
  if (fout) fprintf(fout," dimension m %s\n",ss);
  
  sprintf(ss," %6.2f%s",dens,"%");
  LeftDots(ss,55);
  printf(" density of S %s\n",ss);
  if (fout) fprintf(fout," density of C: %s\n",ss);

  sprintf(ss," %6.2f%s",dens2,"%");
  LeftDots(ss,46);
  printf(" density of S_cholesky %s\n\n",ss);
  if (fout) fprintf(fout," density of S_cholesky: %s\n\n",ss);

  if (fout) fprintf(fout,"\nIteration Information\n");
  if (fout) fprintf(fout,"---------------------\n");
  
  printf("ITER  |P(z)|       POBJ           DOBJ        RGAP     RHO   STEP  ||y|| ");
  if (fout) fprintf(fout,"ITER  |P(z)|       POBJ           DOBJ          RGAP    RHO   STEP  ||y||  ");

  printf("\n-------------------------------");
  printf("-----------------------------------------\n");
  if (fout) fprintf(fout,"\n-------------------------------");
  if (fout) fprintf(fout,"-----------------------------------------\n");
} /* ShowMsgTit */

void PrintIt(sdpdat *sdt,
             int    iter)
{
  double pobj,dobj,rho,rgap,lstp,potential,ynorm;
  char *c1,*c2,*c3;
  
  rho = sdt->rho/(double)(sdt->n+sdt->p);
  rgap = sdt->rgap;
  lstp = sdt->alpha[0]/sdt->pnorm;
  potential = sdt->potential;
  ynorm=sdt->ynorm;

  if (sdt->par.minimize == TRUE)
  { 
    pobj=sdt->pobj;
    dobj=sdt->dobj;
  }
  else
  {
    pobj = -sdt->pobj;
    dobj = -sdt->dobj;
  }

  if(iter==0) c3=" "; else c3 ="";
  if(pobj<0) c1=""; else c1=" ";
  if(dobj<0) c2=""; else c2=" ";

#ifdef PCMACHINE

          
      printf("%s %-3d %5.2f %s %13.6e %s %13.6e"
             " %5.1e %6.2f %4.2f %6.0e \n",
             c3,iter,sdt->pnorm,c1,pobj,c2,dobj,rgap,
             rho,lstp,ynorm);
      if (fout) fprintf(fout,"%s %-3d %6.2f %s %13.6e %s %13.6e"
                   " %6.1e %6.2f %5.2f %6.0e \n",
                   "",iter,sdt->pnorm,c1,pobj,c2,dobj,rgap,
                   rho,lstp,ynorm);
#else
      printf("%s %-3d %6.2f %s %13.6e %s %13.6e "
             " %6.1e  %6.2f %5.2f %6.0e \n",
             c3,iter,sdt->pnorm,"",pobj,"",dobj,rgap,
             rho,lstp,ynorm);
      if (fout) fprintf(fout,"%s %-3d %6.2f %s %13.6e %s %13.6e "
                   " %6.1e  %6.2f %5.2f %6.0e \n",
                  "",iter,sdt->pnorm,"",pobj,"",dobj,rgap,
                   rho,lstp,ynorm);
#endif

} /* ShowMsg */


void PrintEndSDP()
{
  int i;

  for (i=0; i<72; i++)
  {
    printf("-");
    if (fout) fprintf(fout,"-");
  }
  printf("\n\n");
  if (fout) fprintf(fout,"\n\n");
}


void PrintIntObj(int n,double eigObj, double hObj, double rObj, params *par){

  int option;

  option = par->findrank1sol;

  if (par->minimize == FALSE)
  {
    eigObj = -eigObj;
    hObj = -hObj;
    rObj = -rObj;
  }

  printf("\n");
  if (option == 2 || option == 3) {
    printf(" Using rows of primal matrix, best solution:         %8.6f \n",hObj);
    if (fout) fprintf(fout," Using rows of primal matrix, best solution:         %8.6f \n",hObj);
  }  
  if (option == 3 || option ==4){
    printf(" Using %4d random unit vectors, best solution is:   %8.6f \n",
	   n,rObj);
    if (fout) fprintf(fout," Using %4d random unit vectors, best solution is:   %8.6f \n",
	   n,rObj);
  }
  if (option == 1 || option== 3){
    printf(" Using the dominant eigenvector, objective value:    %8.6f \n",eigObj);
    if (fout) fprintf(fout," Using the dominant eigenvector, objective value:    %8.6f \n",eigObj);
  }
  printf("\n"); 
  if (fout) fprintf(fout,"\n"); 
}


void PrintEnd(clock_t tim[])
{
  double tmproc;
  char   ss[LineSize];

  printf("\n");
    
  if (fout) fprintf(fout,"\nTime distribution");
  if (fout) fprintf(fout,"\n-----------------\n");
  
  tmproc=TimeInSec(tim[START],tim[DATAIN]);
  sprintf(ss," %.2f sec",tmproc);
  LeftDots(ss,39);
  printf(" time for initial data input %s\n",ss);
  if (fout) fprintf(fout," time for initial data input %s\n",ss);

  tmproc=TimeInSec(tim[DATAIN],tim[DUALALG]);
  sprintf(ss," %.2f sec",tmproc);
  LeftDots(ss,32);
  printf(" time to solve semidefinite program %s\n",ss);
  if (fout) fprintf(fout," time to solve semidefinite program %s\n",ss);
  
  tmproc=TimeInSec(tim[DUALALG],tim[INTEG]);
  sprintf(ss," %.2f sec",tmproc);
  LeftDots(ss,31);
  printf(" time for vector solution generation %s\n",ss);
  if (fout) fprintf(fout," time for vector solution generation %s\n",ss);

  tmproc=TimeInSec(tim[START],tim[ELAPSED]);
  sprintf(ss," %.2f sec",tmproc);
  LeftDots(ss,40);
  printf(" time for the whole process %s\n",ss);
  if (fout) fprintf(fout," time for the whole process %s\n",ss);

  printf("\n **************************** COPL ENDS ");
  printf("****************************\n\n");  

  if (fout) fprintf(fout,"\n---------------------------  EOF  ");
  if (fout) fprintf(fout,"-------------------------------\n\n");
  
  return;
}/* PrintEnd */


void PrintVecSol(double *bestv, int n, char *outfile, probtype ptype)
/* Save the best answer to a file 'outfile.vsol  */
{
  int i;
  FILE *fp;
  char ss[60];
  
  sprintf(ss,"%s.vsol",outfile);
  
  fp=fopen(ss,"w");
  if (!fp)
    ExitProc(WriteFail,ss);
  
  if (ptype == boxqp || ptype == user){
    for (i=0; i<n; i++){  
      fprintf(fp,"%6.4f \n", bestv[i]);
    }
  } else {
    for (i=0; i<n; i++){  
      fprintf(fp,"%2.0f \n", bestv[i]);
    }
  }
  fclose(fp);
}      


void SaveSolution(double zbar, double *ysol, int m, 
		  char *outfile, int minimize)
     /* Save the y solution  to a file 'outfile.ysol  */
{
  int i;
  FILE *fp;
  
  if (minimize == FALSE) zbar = -zbar;
  
  fp=fopen(outfile,"w");
  if (!fp)
    ExitProc(WriteFail,outfile);

  /*   Print primal objective value
    fprintf(fp,"%10.8f \n",zbar);
  */

  for (i=0; i<m; i++){
    fprintf(fp,"%16.14f \n", ysol[i]);
  }
  
  fclose(fp);
}      








