
#include "SDPdef.h"
void UserRule(double *, double *, int );
 

static double VDot(double *v1, double *v2, int n){
  int i;
  double ans=0.0;

  for (i=0; i<n; i++){  ans = ans+v1[i]*v2[i]; }
  return ans;
}

double EvalObjective( symat  *objM,  
                      double *nvec,  
                      int    n)
/* Evaluate the quadratic function xQx where Q  is a symmetric
   sparse matrix and x is a vector.  This is done by calculating
   the inner product of Q with the outer product of the vector  */
{  double answer=0; 
   int i,j,k,nnzr; 
  
   for (i=0;i<n;i++)
   {
      answer = answer + objM->diag[i] * nvec[i] * nvec[i];
      nnzr = objM->roff[i].nn0;
      for (j=0;j<nnzr;j++)
      {
         k = objM->roff[i].ja[j];
         answer = answer + objM->roff[i].an[j] * nvec[i] * nvec[k] * 2;
      }
   } 
   return (answer);
} 
 
static int cut_comp(const void *e1,const void *e2)
/* Used in qsort routine to sort the structure 'orderVec' by its value  */
{ 
   double d1, d2;
   orderVec *a1,*a2;
   a1=(orderVec*)e1;
   a2=(orderVec*)e2;
   d1 = *(a1->val); 
   d2 = *(a2->val); 
 
   if (d1<d2) 
      return (-1); 
   else if (d1>d2) 
      return (1); 
   return(0); 
}

static orderVec *OrdAlloc(int  len, char *info)
{
  orderVec *r=NULL;
  
  if (len) {
    r=(orderVec*)calloc(len,sizeof(orderVec));
    if (!r) ExitProc(OutOfSpc,info);
  }
  return r;
} /* OrdAlloc */

static void OrdFree(orderVec **x)
{
  orderVec *r=*x;
  
  if (r) {
    free(r);
    *x=NULL;
  }
} /* ordFree */


static int MaxCutRule(double *vhat, double *vsol, int n){
  int i;

  for (i=0;i<n;i++){
    if (vhat[i]>0) vsol[i]=1.0;
    else vsol[i]=-1.0;
  }
  return 0;
}

static int BoxqpRule(double *vhat, double *vsol, int n){
  int i;

  for (i=0;i<n;i++){
    vsol[i] = sign(vhat[i])*fabs(vsol[i]); 
  }
  return 0;
}

static int SizeCutRule(double *vhat, double *vsol, int n, 
		       orderVec *vhatorder, int cut){
  int i;

  for (i=0;i<n;i++){
    vhatorder[i].index=i;
    vhatorder[i].val = &(vhat[i]);
  }
  
  qsort( (void *)vhatorder, n, sizeof(orderVec), cut_comp);
    
  if ( *(vhatorder[n/2].val) > 0){
    for ( i=0;i<(n-cut)/2;i++)
      vsol[vhatorder[i].index] = -1.0;
    for ( i=(n-cut)/2; i<n; i++)
      vsol[vhatorder[i].index] = 1.0;
  }
  
  else{
    for ( i=0;i<(n+cut)/2;i++)
      vsol[vhatorder[i].index] = 1.0;
    for ( i=(n+cut)/2; i<n; i++) 
      vsol[vhatorder[i].index] = -1.0;
  }

  /*  Could also implement for nonbinary variables 
  if ( *(vhatorder[n/2].val) > 0){
    for ( i=0;i<(n-cut)/2;i++)
      vsol[vhatorder[i].index] = -fabs(vsol[vhatorder[i].index]);
    for ( i=(n-cut)/2; i<n; i++)
      vsol[vhatorder[i].index] = fabs(vsol[vhatorder[i].index]);
  }
  
  else{
    for ( i=0;i<(n+cut)/2;i++)
      vsol[vhatorder[i].index] = fabs(vsol[vhatorder[i].index]);
    for ( i=(n+cut)/2; i<n; i++) 
      vsol[vhatorder[i].index] = -fabs(vsol[vhatorder[i].index]);
  }
  */

  return 0;
}

static int MaxCoverRule(double *vhat, double *vsol, int n, sdpdat *sdt)
    /* This routine creates feasible solutions for the stable set problems.
	It assumes the constraints |v_i + v_j + v_n| are written after the
	boolean constraints and checks for violated contraints.  
	If it finds a violated constraint, it will change the sign of either
	v_i or v_j, whichever is farther from v_n.                      */
{
  int i;
  spvec *spv;
  
  for (i=0;i<sdt->n; i++){
    if (vhat[i]<0) vsol[i] = -1;
    else vsol[i] = 1;
  }
  for (i=0;i<sdt->m; i++){ if (sdt->amats->rows[i].nn0 == 3){
    spv = &(sdt->amats->rows[i]);
    if ( (sign(vsol[spv->ja[0]]) == sign(vsol[spv->ja[2]])) && 
	 (sign(vsol[spv->ja[1]]) == sign(vsol[spv->ja[2]])) ){
      if ( (vhat[spv->ja[0]]-vhat[spv->ja[2]]) > 
	   (vhat[spv->ja[1]]-vhat[spv->ja[2]]) ){
	vsol[spv->ja[0]] = -vsol[spv->ja[0]];
      }
      else{
	vsol[spv->ja[1]] = -vsol[spv->ja[1]];
      }
    }
  }}
  

  return 0;
}
  



void GetRandVec(double *randVec, int n) 
/* This routine obtains a vector from the n dimensional
   unit sphere.  Actually, if the vector were to be normalized
   to length one, it would be a randomly chosen vector from the
   unit sphere.                                                 */
   
{  int i; 
   double scal,x;
   scal=pow(2.0,15)-1.0;
   scal = RAND_MAX;
   for (i=0;i<n;i++){ 
     x = (( rand())/scal - .5);
     randVec[i] = tan(3.1415926*x); 
   }
}



void RankReduce(sdpdat *sdt)
/* This routine generates a specific cut or solution to the original, 
   unrelaxed problem. */ 
{
  int i,j=0,m=sdt->m, p=sdt->p, n=sdt->n;
  int  ind,intcut,cutsize,cut=0;
  int method=sdt->par.findrank1sol,useSMW, ndense=sdt->smw->ndense;
  double one=1.0,thisObj;
  double *vec1,*vec2,*vec3,*vhat,*randVec, *vsol,*ei,*val,*xvar;
  double temp, bigeig=0.0, oldeig=0.0; 
  orderVec *vorder;
  spvec spei;
  chfac *tempcf;
  smwTools *smw2;
  probtype ptype=sdt->par.prtype;

  if (sdt->gotpobj == FALSE){
    printf("Sorry, no feasible matrix could be found for the \n"); 
    printf(" primal positive semidefinite program. \n");
    return;
  }

  useSMW = ndense;
  
  spei.nn0 = 1;   spei.an=&one;  spei.ja=&ind;
  vsol    = sdt->vsol;
  vhat    = sdt->vhat;
  vec1    = sdt->wd;
  vec2    = sdt->wd2;

  val     = dAlloc(n,"Insufficient memory to store cut"); 
  randVec = dAlloc(n,"Insufficient memory to store cut"); 
  vec3    = dAlloc(n,"Insufficient memory to store cut");
  ei      = dAlloc(n,"Insufficient memory to store cut");  
  vorder  = OrdAlloc(sdt->n,"Insufficient memory to store cut");
  smw2    = wAlloc(sdt->smw->ndense,sdt->n);  
  
   
  srand(clock());   /*  Used to generate random vectors */
  
  /* Free the large gram matrix, do symbollic work */
  CfcFree(&sdt->mf);
  sdt->gram = NULL;
  sdt->dgram = NULL;


  if (sdt->par.prtype == ecut){
    intcut = 1;
    for (i=0; i<m; i++){
      if (sdt->amats->rows[i].nn0 > intcut){
	intcut = sdt->amats->rows[i].nn0;
	cut = (int) sqrt(fabs(sdt->b[i])+1e-8);
      }
    }
    cut = min(max(0,cut),n);
    cutsize=cut;
    if ((n-cut)%2 == 1) cut ++;
    printf("Trying to satisfy e^T v = %d \n\n",cut);
  }

  if (sdt->par.prtype == maxcover){
    intcut = 0;
    for (i=0; i<m; i++){
      if ( sdt->amats->rows[i].nn0 == 1)
	continue;
      else if ( sdt->amats->rows[i].nn0 == 3){
	if (sdt->amats->rows[i].ja[2] != sdt->n-1){
	  printf("Warning: third nonzero index in constraint %d", i+1);
	  printf(" is inconsistent with this heuristic. \n");
	  intcut = 5;
	}}
      else intcut = sdt->amats->rows[i].nn0;
    }
    if (intcut)
      printf("Warning: problem data doesn't match expectations. \n");
  }
  /* Initialize the vector solution to the the correct magnitude. 
     Look for constraints that bound the diagonal of the primal matrix
     Calculate linear part and subtract it from the rhs */
   
   if (sdt->p > 0){
     xvar = dAlloc(sdt->p,"Insufficient memory to store cut"); 
     AddVecs(1.0,sdt->xmakery,-1.0,sdt->xmakerdy,sdt->ytemp, sdt->m);
     LPVariables(sdt, sdt->ytemp, xvar);
     LPVariables(sdt, sdt->xmakery, sdt->s);
     for (i=0; i<p; i++){
       xvar[i]=xvar[i]/(sdt->s[i]*sdt->s[i])*sdt->xmakerc;
     }
     
   }

   for (i=0; i<n; i++) val[i] = 1.0;

   if (ptype == boxqp || ptype == user){
     for (i=0;i<sdt->m;i++){
       if (sdt->amats->rows[i].nn0 == 1  
	   /* && sdt->aa.amat[i].ja[0] == i  Part2 optional */){
	 if (sdt->p == 0) val[sdt->amats->rows[i].ja[0]] = sqrt(fabs((sdt->b[i])));
	 else{
	   val[sdt->amats->rows[i].ja[0]]=
	     sqrt(fabs( sdt->b[i] - VDotSD(&(sdt->avecs->rows[i]),xvar) ));
	 }
       }  
     }
   }   

   if (sdt->p > 0) dFree(&xvar); 

   
   
   if (method==3 || method == 4){
     /* Use random vectors  to calculate a solution */

     tempcf=sdt->sf;
     sdt->sf = sdt->rf;
     CreateS(sdt, &temp);

     AddVecs(1.0,sdt->xmakery,-1.0,sdt->xmakerdy,sdt->ytemp, sdt->m);
     NumChol(sdt,sdt->ytemp,FALSE);
     RVectorCoeffs(sdt, sdt->ytemp);

     smw2->ndiag = sdt->smw->ndiag;
     for (i=0; i<ndense+smw2->ndiag; i++){
       smw2->v2[i] = sdt->smw->v2[i];
     }
     for (i=ndense+smw2->ndiag; i<2*ndense; i++){
       smw2->v2[i]=0.0;
     }

     for (i=0; i<ndense; i++){
       for (j=0; j<sdt->n; j++){
      	 smw2->upd[i][j]=sdt->smw->upd[i][j];
       }
     }
     for (i=ndense; i<2*ndense; i++){
       for (j=0; j<sdt->n; j++) smw2->upd[i][j]=0.0;
       if ( i<ndense + smw2->ndiag ){
	 smw2->upd[i][sdt->smw->ndInd[i-ndense]]=1.0;
       }
     }       

     sdt->rf = sdt->sf;
     sdt->sf= tempcf;
   }

   NumChol(sdt,sdt->xmakery,FALSE);
   if (CheckPSD(sdt,sdt->xmakery,sdt->sf)) printf("Nearly Singular \n");
   if (useSMW) MakeSMWmat(sdt,sdt->xmakery);


   if (method==3 || method == 4){
     /* Use randomized method to calculate a solution */

     for (i=0; i<n; i++){
       /* Calculate vhat */
       for (j=0;j<n;j++) vsol[j] = val[j];
       GetRandVec(randVec,n);
       /* For user function, normalize the vector */
       if (ptype==user){
	 temp=sqrt(sdt->xmakerc/VDot(randVec,randVec,n));
	 for (j=0; j<n; j++) randVec[j]=randVec[j]*temp;
       }


       for (j=0; j<ndense+smw2->ndiag; j++){
	 temp=smw2->v2[j]*VDot(randVec,smw2->upd[j],sdt->n);
	 AddVecs(1.0, randVec, temp, smw2->upd[j], randVec, n);
       }

       DoubleCopy(randVec,vec1,n);
       GetUhat(sdt->rf,vec1,vec2);
       DoubleCopy(vec2,vec1,n);

       ChlSolve(sdt->sf,vec2, vec3);
       if (useSMW) SMWupdate(sdt->smw,NULL,vec1,vec3,n,FALSE);
       DoubleCopy(vec3, vhat, n);


       /* Use vhat to find a solution */
       if (ptype == maxcut ) MaxCutRule(vhat, vsol, n);
       else if (ptype == boxqp ) BoxqpRule(vhat, vsol, n);
       else if (ptype==maxcover) MaxCoverRule(vhat, vsol, n, sdt);
       else if (ptype==ecut) SizeCutRule(vhat, vsol, n, vorder, cutsize);
       else if (ptype==user) UserRule(vhat,vsol,n);
       else MaxCutRule(vhat, vsol, n);


       thisObj = EvalObjective(sdt->cmat,vsol,n);
       if (i==0 || thisObj < sdt->bestRObj){
	 sdt->bestRObj=thisObj; 
	 if (thisObj < sdt->bestObj){
	   sdt->bestObj = thisObj;
	   DoubleCopy(vsol, sdt->bestv, n);
	 }
       }
     }
   }



   if (method == 2 || method == 3){
     /* Use the rows of the primal solution X to find vector solution */
     for (i=0; i<n; i++){       /* Calculate row i of X */

       memset(ei,0,sizeof(double)*n);
       ei[i]=1.0 ;
       *(spei.ja)=i;
       for (j=0;j<n;j++) vsol[j] = val[j];

       ChlSolve(sdt->sf, ei, vec1);
       if (useSMW) SMWupdate(sdt->smw,&spei,NULL,vec1,n,TRUE);

       ATdyTv(sdt,sdt->xmakerdy,vec1, vec2);

       DoubleCopy(vec2,ei,n);

       ChlSolve(sdt->sf, ei, vec3);
       if (useSMW) SMWupdate(sdt->smw,NULL,vec2,vec3,n,FALSE);
       AddVecs(1.0, vec1, 1.0, vec3, vhat, n);

       if (ptype==user){for (j=0; j<n; j++) vhat[i]=vhat[i]*sdt->xmakerc;}

       /* Use the row to find a solution */
       if (ptype == maxcut ) MaxCutRule(vhat, vsol, n);
       else if (ptype == boxqp ) BoxqpRule(vhat, vsol, n);
       else if (ptype==maxcover) MaxCoverRule(vhat, vsol, n, sdt);
       else if (ptype==ecut) SizeCutRule(vhat, vsol, n, vorder, cutsize);
       else if (ptype==user) UserRule(vhat,vsol,n);
       else MaxCutRule(vhat, vsol, n);
	 

       thisObj = EvalObjective(sdt->cmat,vsol,n); 
       if (i==0 || thisObj < sdt->bestHObj){ 
	 sdt->bestHObj=thisObj;
	 if (thisObj < sdt->bestObj){
	   sdt->bestObj=thisObj;
	   DoubleCopy(vsol, sdt->bestv, n);
	 }
       }
     }
   }

   if (method == 1 || method==3){

     GetRandVec(vhat,n);
     temp=sqrt(fabs(VDot(vhat,vhat,n)));
     oldeig = 0; bigeig=0;

     for (j=0; j<n/2+1; j++){
       if (oldeig > 0 && bigeig/oldeig > 0.9999) break;

       for (i=0; i<n; i++) vhat[i]=vhat[i]/(temp);

       ChlSolve(sdt->sf, vhat, vec1);
       if (useSMW) SMWupdate(sdt->smw,&spei,NULL,vec1,n,TRUE);
       
       ATdyTv(sdt,sdt->xmakerdy,vec1, vec2);

       DoubleCopy(vec2,ei,n);      

       ChlSolve(sdt->sf, ei, vec3);
       if (useSMW) SMWupdate(sdt->smw,NULL,vec2,vec3,n,FALSE);
       AddVecs(1.0, vec1, 1.0, vec3, vhat, n);

       oldeig = bigeig;
       temp=fabs(VDot(vhat,vhat,n));
       bigeig= sqrt(temp*sdt->xmakerc); temp=sqrt(temp);
     }

     if (ptype == maxcut ) MaxCutRule(vhat, vsol, n);
     else if (ptype == boxqp ) BoxqpRule(vhat, vsol, n);
     else if (ptype==maxcover) MaxCoverRule(vhat, vsol, n, sdt);
     else if (ptype==ecut) SizeCutRule(vhat, vsol, n, vorder, cutsize);
     else if (ptype==user) UserRule(vhat,vsol,n);
     else MaxCutRule(vhat, vsol, n);

     sdt->eigObj = EvalObjective(sdt->cmat,vsol,n); 

     if (sdt->eigObj < sdt->bestObj){
       sdt->bestObj=sdt->eigObj;
       DoubleCopy(vsol, sdt->bestv, n);
     }
   }

   PrintIntObj(n,sdt->eigObj, sdt->bestHObj, sdt->bestRObj, &(sdt->par) ); 

   if (sdt->par.saverank1sol) 
     PrintVecSol(sdt->bestv,n,sdt->ss, sdt->par.prtype);
   
   dFree(&val); 
   dFree(&randVec); 
   OrdFree(&vorder);
   dFree(&vec3);
   dFree(&ei);
   wFree(&smw2);
} 


void ATdyTv(sdpdat *sdt, double *co, double *v1, double* ans){
/* This function computes the vector ans=A^T(co)*v1.  Recall
   that the matrices A_i have a rank one structure.         */

  int i,j,m=sdt->m,n=sdt->n,nnz;
  double temp;
  spvec *svec;

  for (i=0; i<n; i++){
    ans[i]= 0.0;
  }

  for (i=0; i<m; i++){
    svec = &(sdt->amats->rows[i]);
    temp = VDotSD(svec, v1);
    nnz = svec->nn0;
    for (j=0; j<nnz; j++){
      ans[svec->ja[j]] += temp*co[i]*(svec->an[j]);
    }
  }
}







